@@ -30,8 +30,10 @@
 #include "../objects/object_factory.h"
 #include "../objects/object_type.h"
 #include "../objects/conic_types.h"
+#include "../objects/line_type.h"
 #include "../objects/intersection_types.h"
 #include "../objects/object_imp.h"
+#include "../objects/point_imp.h"
 #include "../objects/bogus_imp.h"
 #include "../objects/conic_imp.h"
 #include "../objects/other_type.h"
@@ -46,7 +48,7 @@
 
 ConicRadicalConstructor::ConicRadicalConstructor()
   : StandardConstructorBase(
-    I18N_NOOP( "Radical line for conics" ),
+    I18N_NOOP( "Radical Line for Conics" ),
     I18N_NOOP( "The lines constructed through the intersections "
                "of two conics.  This is also defined for "
                "non-intersecting conics." ),
@@ -105,7 +107,7 @@ Objects ConicRadicalConstructor::build( const Objects& os, KigDocument&, KigWidg
 
 static const struct ArgParser::spec argsspecpp[] =
 {
-  { ObjectImp::ID_PointImp, I18N_NOOP( "Moving point" ) },
+  { ObjectImp::ID_PointImp, I18N_NOOP( "Moving Point" ) },
   { ObjectImp::ID_PointImp, I18N_NOOP( "following" ) }
 };
 
@@ -184,8 +186,8 @@ QString LocusConstructor::useText( const Object& o, const Objects& os,
        static_cast<const RealObject&>( o ).type()->inherits( ObjectType::ID_ConstrainedPointType ) &&
        ( os.empty() || !os[0]->inherits( Object::ID_RealObject ) ||
          !static_cast<const RealObject*>( os[0] )->type()->inherits( ObjectType::ID_ConstrainedPointType ) )
-    ) return i18n( "Moving point" );
-  else return i18n( "Dependent point" );
+    ) return i18n( "Moving Point" );
+  else return i18n( "Dependent Point" );
 }
 
 void ConicRadicalConstructor::plug( KigDocument*, KigGUIAction* )
@@ -207,8 +209,8 @@ bool LocusConstructor::isTransform() const
 }
 
 static const ArgParser::spec argsspectc[] = {
-  { ObjectImp::ID_ConicImp, I18N_NOOP( "Intersect with this circle" ) },
-  { ObjectImp::ID_ConicImp, I18N_NOOP( "Intersect with this circle" ) }
+  { ObjectImp::ID_ConicImp, "" },
+  { ObjectImp::ID_ConicImp, "" }
 };
 
 ConicConicIntersectionConstructor::ConicConicIntersectionConstructor()
@@ -291,17 +293,6 @@ bool ConicConicIntersectionConstructor::isTransform() const
   return false;
 }
 
-QString ConicConicIntersectionConstructor::useText(
-  const Object& o, const Objects&,
-  const KigDocument&, const KigWidget& ) const
-{
-  if ( o.hasimp( ObjectImp::ID_CircleImp ) )
-    return i18n( "Intersect with this circle" );
-  else if ( o.hasimp( ObjectImp::ID_ConicImp ) )
-    return i18n( "Intersect with this conic" );
-  else assert( false );
-}
-
 ConicLineIntersectionConstructor::ConicLineIntersectionConstructor()
   : MultiObjectTypeConstructor(
     ConicLineIntersectionType::instance(),
@@ -314,26 +305,13 @@ ConicLineIntersectionConstructor::~ConicLineIntersectionConstructor()
 {
 }
 
-QString ConicLineIntersectionConstructor::useText(
-  const Object& o, const Objects&,
-  const KigDocument&, const KigWidget& ) const
-{
-  if ( o.hasimp( ObjectImp::ID_CircleImp ) )
-    return i18n( "Intersect with this circle" );
-  else if ( o.hasimp( ObjectImp::ID_ConicImp ) )
-    return i18n( "Intersect with this conic" );
-  else if ( o.hasimp( ObjectImp::ID_LineImp ) )
-    return i18n( "Intersect with this line" );
-  else assert( false );
-}
-
 QString ConicRadicalConstructor::useText( const Object& o, const Objects&,
                                           const KigDocument&, const KigWidget& ) const
 {
   if ( o.hasimp( ObjectImp::ID_CircleImp ) )
-    return i18n( "Construct the radical lines of this circle" );
+    return i18n( "Construct the Radical Lines of This Circle" );
   else
-    return i18n( "Construct the radical lines of this conic" );
+    return i18n( "Construct the Radical Lines of This Conic" );
 }
 
 GenericIntersectionConstructor::GenericIntersectionConstructor()
@@ -378,3 +356,73 @@ bool GenericIntersectionConstructor::isIntersection() const
 {
   return true;
 }
+
+QString GenericIntersectionConstructor::useText(
+  const Object& o, const Objects&,
+  const KigDocument&, const KigWidget& ) const
+{
+  if ( o.hasimp( ObjectImp::ID_CircleImp ) )
+    return i18n( "Intersect with This Circle" );
+  else if ( o.hasimp( ObjectImp::ID_ConicImp ) )
+    return i18n( "Intersect with This Conic" );
+  else if ( o.hasimp( ObjectImp::ID_LineImp ) )
+    return i18n( "Intersect with This Line" );
+  else if ( o.hasimp( ObjectImp::ID_CubicImp ) )
+    return i18n( "Intersect with This Cubic" );
+  else assert(
+    false );
+}
+
+static const ArgParser::spec argsspecMidPointOfTwoPoints[] =
+{
+  { ObjectImp::ID_PointImp, I18N_NOOP( "Construct the midpoint of this point" ) },
+  { ObjectImp::ID_PointImp, I18N_NOOP( "Construct the midpoint of this point" ) }
+};
+
+MidPointOfTwoPointsConstructor::MidPointOfTwoPointsConstructor()
+  : StandardConstructorBase( "", "", "", mparser ),
+    mparser( argsspecMidPointOfTwoPoints, 2 )
+{
+}
+
+MidPointOfTwoPointsConstructor::~MidPointOfTwoPointsConstructor()
+{
+}
+
+void MidPointOfTwoPointsConstructor::drawprelim(
+  KigPainter& p, const Objects& parents,
+  const KigDocument& ) const
+{
+  if ( parents.size() != 2 ) return;
+  assert( parents[0]->hasimp( ObjectImp::ID_PointImp ) );
+  assert( parents[1]->hasimp( ObjectImp::ID_PointImp ) );
+  const Coordinate m =
+    ( static_cast<const PointImp*>( parents[0]->imp() )->coordinate() +
+      static_cast<const PointImp*>( parents[1]->imp() )->coordinate() ) / 2;
+  PointImp( m ).draw( p );
+}
+
+Objects MidPointOfTwoPointsConstructor::build(
+  const Objects& os, KigDocument& d, KigWidget& ) const
+{
+  RealObject* seg = new RealObject( SegmentABType::instance(), os );
+  seg->setShown( false );
+  seg->calc( d );
+  int index = seg->imp()->propertiesInternalNames().findIndex( "mid-point" );
+  assert( index != -1 );
+  PropertyObject* prop = new PropertyObject( seg, index );
+  RealObject* point = new RealObject( CopyObjectType::instance(), Objects( prop ) );
+  Objects ret( seg );
+  ret.push_back( prop );
+  ret.push_back( point );
+  return ret;
+}
+
+void MidPointOfTwoPointsConstructor::plug( KigDocument*, KigGUIAction* )
+{
+}
+
+bool MidPointOfTwoPointsConstructor::isTransform() const
+{
+  return false;
+}
